# Generated from XeasyPKParser.g4 by ANTLR 4.13.0
# encoding: utf-8
from antlr4 import *
from io import StringIO
import sys
if sys.version_info[1] > 5:
	from typing import TextIO
else:
	from typing.io import TextIO

def serializedATN():
    return [
        4,1,40,223,2,0,7,0,2,1,7,1,2,2,7,2,2,3,7,3,2,4,7,4,2,5,7,5,2,6,7,
        6,2,7,7,7,2,8,7,8,2,9,7,9,2,10,7,10,2,11,7,11,2,12,7,12,2,13,7,13,
        2,14,7,14,2,15,7,15,1,0,3,0,34,8,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,
        0,1,0,5,0,45,8,0,10,0,12,0,48,9,0,1,0,1,0,1,1,1,1,1,1,1,1,1,2,1,
        2,1,2,1,2,1,3,1,3,1,3,1,3,1,3,1,4,1,4,1,4,1,4,1,5,1,5,4,5,71,8,5,
        11,5,12,5,72,1,5,1,5,1,6,1,6,4,6,79,8,6,11,6,12,6,80,1,6,1,6,1,7,
        4,7,86,8,7,11,7,12,7,87,1,8,1,8,1,8,1,8,1,8,1,8,1,8,1,8,1,8,1,8,
        1,8,1,8,3,8,102,8,8,1,8,1,8,3,8,106,8,8,1,8,1,8,1,8,3,8,111,8,8,
        1,8,1,8,3,8,115,8,8,5,8,117,8,8,10,8,12,8,120,9,8,1,9,4,9,123,8,
        9,11,9,12,9,124,1,10,1,10,1,10,1,10,1,10,1,10,1,10,1,10,1,10,1,10,
        1,10,1,10,1,10,1,10,3,10,141,8,10,1,10,1,10,3,10,145,8,10,1,10,1,
        10,1,10,1,10,3,10,151,8,10,1,10,1,10,3,10,155,8,10,5,10,157,8,10,
        10,10,12,10,160,9,10,1,11,4,11,163,8,11,11,11,12,11,164,1,12,1,12,
        1,12,1,12,1,12,1,12,1,12,1,12,1,12,1,12,1,12,1,12,1,12,1,12,1,12,
        1,12,3,12,183,8,12,1,12,1,12,3,12,187,8,12,1,12,1,12,1,12,1,12,1,
        12,3,12,194,8,12,1,12,1,12,3,12,198,8,12,5,12,200,8,12,10,12,12,
        12,203,9,12,1,13,1,13,1,14,1,14,1,14,3,14,210,8,14,3,14,212,8,14,
        1,15,1,15,5,15,216,8,15,10,15,12,15,219,9,15,1,15,1,15,1,15,0,0,
        16,0,2,4,6,8,10,12,14,16,18,20,22,24,26,28,30,0,2,2,0,7,9,13,13,
        1,1,40,40,239,0,33,1,0,0,0,2,51,1,0,0,0,4,55,1,0,0,0,6,59,1,0,0,
        0,8,64,1,0,0,0,10,68,1,0,0,0,12,76,1,0,0,0,14,85,1,0,0,0,16,89,1,
        0,0,0,18,122,1,0,0,0,20,126,1,0,0,0,22,162,1,0,0,0,24,166,1,0,0,
        0,26,204,1,0,0,0,28,211,1,0,0,0,30,213,1,0,0,0,32,34,5,15,0,0,33,
        32,1,0,0,0,33,34,1,0,0,0,34,46,1,0,0,0,35,45,3,2,1,0,36,45,3,4,2,
        0,37,45,3,6,3,0,38,45,3,8,4,0,39,45,3,10,5,0,40,45,3,12,6,0,41,45,
        3,14,7,0,42,45,3,18,9,0,43,45,3,22,11,0,44,35,1,0,0,0,44,36,1,0,
        0,0,44,37,1,0,0,0,44,38,1,0,0,0,44,39,1,0,0,0,44,40,1,0,0,0,44,41,
        1,0,0,0,44,42,1,0,0,0,44,43,1,0,0,0,45,48,1,0,0,0,46,44,1,0,0,0,
        46,47,1,0,0,0,47,49,1,0,0,0,48,46,1,0,0,0,49,50,5,0,0,1,50,1,1,0,
        0,0,51,52,5,1,0,0,52,53,5,19,0,0,53,54,5,21,0,0,54,3,1,0,0,0,55,
        56,5,2,0,0,56,57,5,22,0,0,57,58,5,24,0,0,58,5,1,0,0,0,59,60,5,3,
        0,0,60,61,5,25,0,0,61,62,5,26,0,0,62,63,5,28,0,0,63,7,1,0,0,0,64,
        65,5,4,0,0,65,66,5,29,0,0,66,67,5,31,0,0,67,9,1,0,0,0,68,70,5,5,
        0,0,69,71,5,32,0,0,70,69,1,0,0,0,71,72,1,0,0,0,72,70,1,0,0,0,72,
        73,1,0,0,0,73,74,1,0,0,0,74,75,5,34,0,0,75,11,1,0,0,0,76,78,5,6,
        0,0,77,79,5,35,0,0,78,77,1,0,0,0,79,80,1,0,0,0,80,78,1,0,0,0,80,
        81,1,0,0,0,81,82,1,0,0,0,82,83,5,37,0,0,83,13,1,0,0,0,84,86,3,16,
        8,0,85,84,1,0,0,0,86,87,1,0,0,0,87,85,1,0,0,0,87,88,1,0,0,0,88,15,
        1,0,0,0,89,90,5,7,0,0,90,91,5,8,0,0,91,92,5,8,0,0,92,93,5,7,0,0,
        93,94,5,13,0,0,94,95,3,26,13,0,95,96,3,26,13,0,96,97,5,13,0,0,97,
        98,5,7,0,0,98,99,3,28,14,0,99,101,3,28,14,0,100,102,5,7,0,0,101,
        100,1,0,0,0,101,102,1,0,0,0,102,105,1,0,0,0,103,106,5,15,0,0,104,
        106,3,30,15,0,105,103,1,0,0,0,105,104,1,0,0,0,106,118,1,0,0,0,107,
        108,3,28,14,0,108,110,3,28,14,0,109,111,5,7,0,0,110,109,1,0,0,0,
        110,111,1,0,0,0,111,114,1,0,0,0,112,115,5,15,0,0,113,115,3,30,15,
        0,114,112,1,0,0,0,114,113,1,0,0,0,115,117,1,0,0,0,116,107,1,0,0,
        0,117,120,1,0,0,0,118,116,1,0,0,0,118,119,1,0,0,0,119,17,1,0,0,0,
        120,118,1,0,0,0,121,123,3,20,10,0,122,121,1,0,0,0,123,124,1,0,0,
        0,124,122,1,0,0,0,124,125,1,0,0,0,125,19,1,0,0,0,126,127,5,7,0,0,
        127,128,5,8,0,0,128,129,5,8,0,0,129,130,5,8,0,0,130,131,5,7,0,0,
        131,132,5,13,0,0,132,133,3,26,13,0,133,134,3,26,13,0,134,135,5,13,
        0,0,135,136,5,7,0,0,136,137,3,28,14,0,137,138,3,28,14,0,138,140,
        3,28,14,0,139,141,5,7,0,0,140,139,1,0,0,0,140,141,1,0,0,0,141,144,
        1,0,0,0,142,145,5,15,0,0,143,145,3,30,15,0,144,142,1,0,0,0,144,143,
        1,0,0,0,145,158,1,0,0,0,146,147,3,28,14,0,147,148,3,28,14,0,148,
        150,3,28,14,0,149,151,5,7,0,0,150,149,1,0,0,0,150,151,1,0,0,0,151,
        154,1,0,0,0,152,155,5,15,0,0,153,155,3,30,15,0,154,152,1,0,0,0,154,
        153,1,0,0,0,155,157,1,0,0,0,156,146,1,0,0,0,157,160,1,0,0,0,158,
        156,1,0,0,0,158,159,1,0,0,0,159,21,1,0,0,0,160,158,1,0,0,0,161,163,
        3,24,12,0,162,161,1,0,0,0,163,164,1,0,0,0,164,162,1,0,0,0,164,165,
        1,0,0,0,165,23,1,0,0,0,166,167,5,7,0,0,167,168,5,8,0,0,168,169,5,
        8,0,0,169,170,5,8,0,0,170,171,5,8,0,0,171,172,5,7,0,0,172,173,5,
        13,0,0,173,174,3,26,13,0,174,175,3,26,13,0,175,176,5,13,0,0,176,
        177,5,7,0,0,177,178,3,28,14,0,178,179,3,28,14,0,179,180,3,28,14,
        0,180,182,3,28,14,0,181,183,5,7,0,0,182,181,1,0,0,0,182,183,1,0,
        0,0,183,186,1,0,0,0,184,187,5,15,0,0,185,187,3,30,15,0,186,184,1,
        0,0,0,186,185,1,0,0,0,187,201,1,0,0,0,188,189,3,28,14,0,189,190,
        3,28,14,0,190,191,3,28,14,0,191,193,3,28,14,0,192,194,5,7,0,0,193,
        192,1,0,0,0,193,194,1,0,0,0,194,197,1,0,0,0,195,198,5,15,0,0,196,
        198,3,30,15,0,197,195,1,0,0,0,197,196,1,0,0,0,198,200,1,0,0,0,199,
        188,1,0,0,0,200,203,1,0,0,0,201,199,1,0,0,0,201,202,1,0,0,0,202,
        25,1,0,0,0,203,201,1,0,0,0,204,205,7,0,0,0,205,27,1,0,0,0,206,212,
        5,7,0,0,207,209,5,13,0,0,208,210,5,7,0,0,209,208,1,0,0,0,209,210,
        1,0,0,0,210,212,1,0,0,0,211,206,1,0,0,0,211,207,1,0,0,0,212,29,1,
        0,0,0,213,217,5,10,0,0,214,216,5,38,0,0,215,214,1,0,0,0,216,219,
        1,0,0,0,217,215,1,0,0,0,217,218,1,0,0,0,218,220,1,0,0,0,219,217,
        1,0,0,0,220,221,7,1,0,0,221,31,1,0,0,0,26,33,44,46,72,80,87,101,
        105,110,114,118,124,140,144,150,154,158,164,182,186,193,197,201,
        209,211,217
    ]

class XeasyPKParser ( Parser ):

    grammarFileName = "XeasyPKParser.g4"

    atn = ATNDeserializer().deserialize(serializedATN())

    decisionsToDFA = [ DFA(ds, i) for i, ds in enumerate(atn.decisionToState) ]

    sharedContextCache = PredictionContextCache()

    literalNames = [ "<INVALID>", "<INVALID>", "'#FORMAT'", "'#INAME'", 
                     "'#CYANAFORMAT'", "'#SPECTRUM'", "'#TOLERANCE'" ]

    symbolicNames = [ "<INVALID>", "Num_of_dim", "Format", "Iname", "Cyana_format", 
                      "Spectrum", "Tolerance", "Integer", "Float", "Real", 
                      "COMMENT", "EXCLM_COMMENT", "SMCLN_COMMENT", "Simple_name", 
                      "SPACE", "RETURN", "ENCLOSE_COMMENT", "SECTION_COMMENT", 
                      "LINE_COMMENT", "Integer_ND", "SPACE_ND", "RETURN_ND", 
                      "Simple_name_FO", "SPACE_FO", "RETURN_FO", "Integer_IN", 
                      "Simple_name_IN", "SPACE_IN", "RETURN_IN", "Simple_name_CY", 
                      "SPACE_CY", "RETURN_CY", "Simple_name_SP", "SPACE_SP", 
                      "RETURN_SP", "Float_TO", "TOACE_TO", "RETURN_TO", 
                      "Any_name", "SPACE_CM", "RETURN_CM" ]

    RULE_xeasy_pk = 0
    RULE_dimension = 1
    RULE_format = 2
    RULE_iname = 3
    RULE_cyana_format = 4
    RULE_spectrum = 5
    RULE_tolerance = 6
    RULE_peak_list_2d = 7
    RULE_peak_2d = 8
    RULE_peak_list_3d = 9
    RULE_peak_3d = 10
    RULE_peak_list_4d = 11
    RULE_peak_4d = 12
    RULE_number = 13
    RULE_assign = 14
    RULE_comment = 15

    ruleNames =  [ "xeasy_pk", "dimension", "format", "iname", "cyana_format", 
                   "spectrum", "tolerance", "peak_list_2d", "peak_2d", "peak_list_3d", 
                   "peak_3d", "peak_list_4d", "peak_4d", "number", "assign", 
                   "comment" ]

    EOF = Token.EOF
    Num_of_dim=1
    Format=2
    Iname=3
    Cyana_format=4
    Spectrum=5
    Tolerance=6
    Integer=7
    Float=8
    Real=9
    COMMENT=10
    EXCLM_COMMENT=11
    SMCLN_COMMENT=12
    Simple_name=13
    SPACE=14
    RETURN=15
    ENCLOSE_COMMENT=16
    SECTION_COMMENT=17
    LINE_COMMENT=18
    Integer_ND=19
    SPACE_ND=20
    RETURN_ND=21
    Simple_name_FO=22
    SPACE_FO=23
    RETURN_FO=24
    Integer_IN=25
    Simple_name_IN=26
    SPACE_IN=27
    RETURN_IN=28
    Simple_name_CY=29
    SPACE_CY=30
    RETURN_CY=31
    Simple_name_SP=32
    SPACE_SP=33
    RETURN_SP=34
    Float_TO=35
    TOACE_TO=36
    RETURN_TO=37
    Any_name=38
    SPACE_CM=39
    RETURN_CM=40

    def __init__(self, input:TokenStream, output:TextIO = sys.stdout):
        super().__init__(input, output)
        self.checkVersion("4.13.0")
        self._interp = ParserATNSimulator(self, self.atn, self.decisionsToDFA, self.sharedContextCache)
        self._predicates = None




    class Xeasy_pkContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def EOF(self):
            return self.getToken(XeasyPKParser.EOF, 0)

        def RETURN(self):
            return self.getToken(XeasyPKParser.RETURN, 0)

        def dimension(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(XeasyPKParser.DimensionContext)
            else:
                return self.getTypedRuleContext(XeasyPKParser.DimensionContext,i)


        def format_(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(XeasyPKParser.FormatContext)
            else:
                return self.getTypedRuleContext(XeasyPKParser.FormatContext,i)


        def iname(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(XeasyPKParser.InameContext)
            else:
                return self.getTypedRuleContext(XeasyPKParser.InameContext,i)


        def cyana_format(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(XeasyPKParser.Cyana_formatContext)
            else:
                return self.getTypedRuleContext(XeasyPKParser.Cyana_formatContext,i)


        def spectrum(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(XeasyPKParser.SpectrumContext)
            else:
                return self.getTypedRuleContext(XeasyPKParser.SpectrumContext,i)


        def tolerance(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(XeasyPKParser.ToleranceContext)
            else:
                return self.getTypedRuleContext(XeasyPKParser.ToleranceContext,i)


        def peak_list_2d(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(XeasyPKParser.Peak_list_2dContext)
            else:
                return self.getTypedRuleContext(XeasyPKParser.Peak_list_2dContext,i)


        def peak_list_3d(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(XeasyPKParser.Peak_list_3dContext)
            else:
                return self.getTypedRuleContext(XeasyPKParser.Peak_list_3dContext,i)


        def peak_list_4d(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(XeasyPKParser.Peak_list_4dContext)
            else:
                return self.getTypedRuleContext(XeasyPKParser.Peak_list_4dContext,i)


        def getRuleIndex(self):
            return XeasyPKParser.RULE_xeasy_pk

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterXeasy_pk" ):
                listener.enterXeasy_pk(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitXeasy_pk" ):
                listener.exitXeasy_pk(self)




    def xeasy_pk(self):

        localctx = XeasyPKParser.Xeasy_pkContext(self, self._ctx, self.state)
        self.enterRule(localctx, 0, self.RULE_xeasy_pk)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 33
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==15:
                self.state = 32
                self.match(XeasyPKParser.RETURN)


            self.state = 46
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while (((_la) & ~0x3f) == 0 and ((1 << _la) & 254) != 0):
                self.state = 44
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,1,self._ctx)
                if la_ == 1:
                    self.state = 35
                    self.dimension()
                    pass

                elif la_ == 2:
                    self.state = 36
                    self.format_()
                    pass

                elif la_ == 3:
                    self.state = 37
                    self.iname()
                    pass

                elif la_ == 4:
                    self.state = 38
                    self.cyana_format()
                    pass

                elif la_ == 5:
                    self.state = 39
                    self.spectrum()
                    pass

                elif la_ == 6:
                    self.state = 40
                    self.tolerance()
                    pass

                elif la_ == 7:
                    self.state = 41
                    self.peak_list_2d()
                    pass

                elif la_ == 8:
                    self.state = 42
                    self.peak_list_3d()
                    pass

                elif la_ == 9:
                    self.state = 43
                    self.peak_list_4d()
                    pass


                self.state = 48
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 49
            self.match(XeasyPKParser.EOF)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class DimensionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Num_of_dim(self):
            return self.getToken(XeasyPKParser.Num_of_dim, 0)

        def Integer_ND(self):
            return self.getToken(XeasyPKParser.Integer_ND, 0)

        def RETURN_ND(self):
            return self.getToken(XeasyPKParser.RETURN_ND, 0)

        def getRuleIndex(self):
            return XeasyPKParser.RULE_dimension

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDimension" ):
                listener.enterDimension(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDimension" ):
                listener.exitDimension(self)




    def dimension(self):

        localctx = XeasyPKParser.DimensionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 2, self.RULE_dimension)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 51
            self.match(XeasyPKParser.Num_of_dim)
            self.state = 52
            self.match(XeasyPKParser.Integer_ND)
            self.state = 53
            self.match(XeasyPKParser.RETURN_ND)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class FormatContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Format(self):
            return self.getToken(XeasyPKParser.Format, 0)

        def Simple_name_FO(self):
            return self.getToken(XeasyPKParser.Simple_name_FO, 0)

        def RETURN_FO(self):
            return self.getToken(XeasyPKParser.RETURN_FO, 0)

        def getRuleIndex(self):
            return XeasyPKParser.RULE_format

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterFormat" ):
                listener.enterFormat(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitFormat" ):
                listener.exitFormat(self)




    def format_(self):

        localctx = XeasyPKParser.FormatContext(self, self._ctx, self.state)
        self.enterRule(localctx, 4, self.RULE_format)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 55
            self.match(XeasyPKParser.Format)
            self.state = 56
            self.match(XeasyPKParser.Simple_name_FO)
            self.state = 57
            self.match(XeasyPKParser.RETURN_FO)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class InameContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Iname(self):
            return self.getToken(XeasyPKParser.Iname, 0)

        def Integer_IN(self):
            return self.getToken(XeasyPKParser.Integer_IN, 0)

        def Simple_name_IN(self):
            return self.getToken(XeasyPKParser.Simple_name_IN, 0)

        def RETURN_IN(self):
            return self.getToken(XeasyPKParser.RETURN_IN, 0)

        def getRuleIndex(self):
            return XeasyPKParser.RULE_iname

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterIname" ):
                listener.enterIname(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitIname" ):
                listener.exitIname(self)




    def iname(self):

        localctx = XeasyPKParser.InameContext(self, self._ctx, self.state)
        self.enterRule(localctx, 6, self.RULE_iname)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 59
            self.match(XeasyPKParser.Iname)
            self.state = 60
            self.match(XeasyPKParser.Integer_IN)
            self.state = 61
            self.match(XeasyPKParser.Simple_name_IN)
            self.state = 62
            self.match(XeasyPKParser.RETURN_IN)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Cyana_formatContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Cyana_format(self):
            return self.getToken(XeasyPKParser.Cyana_format, 0)

        def Simple_name_CY(self):
            return self.getToken(XeasyPKParser.Simple_name_CY, 0)

        def RETURN_CY(self):
            return self.getToken(XeasyPKParser.RETURN_CY, 0)

        def getRuleIndex(self):
            return XeasyPKParser.RULE_cyana_format

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterCyana_format" ):
                listener.enterCyana_format(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitCyana_format" ):
                listener.exitCyana_format(self)




    def cyana_format(self):

        localctx = XeasyPKParser.Cyana_formatContext(self, self._ctx, self.state)
        self.enterRule(localctx, 8, self.RULE_cyana_format)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 64
            self.match(XeasyPKParser.Cyana_format)
            self.state = 65
            self.match(XeasyPKParser.Simple_name_CY)
            self.state = 66
            self.match(XeasyPKParser.RETURN_CY)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class SpectrumContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Spectrum(self):
            return self.getToken(XeasyPKParser.Spectrum, 0)

        def RETURN_SP(self):
            return self.getToken(XeasyPKParser.RETURN_SP, 0)

        def Simple_name_SP(self, i:int=None):
            if i is None:
                return self.getTokens(XeasyPKParser.Simple_name_SP)
            else:
                return self.getToken(XeasyPKParser.Simple_name_SP, i)

        def getRuleIndex(self):
            return XeasyPKParser.RULE_spectrum

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSpectrum" ):
                listener.enterSpectrum(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSpectrum" ):
                listener.exitSpectrum(self)




    def spectrum(self):

        localctx = XeasyPKParser.SpectrumContext(self, self._ctx, self.state)
        self.enterRule(localctx, 10, self.RULE_spectrum)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 68
            self.match(XeasyPKParser.Spectrum)
            self.state = 70 
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while True:
                self.state = 69
                self.match(XeasyPKParser.Simple_name_SP)
                self.state = 72 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if not (_la==32):
                    break

            self.state = 74
            self.match(XeasyPKParser.RETURN_SP)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ToleranceContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Tolerance(self):
            return self.getToken(XeasyPKParser.Tolerance, 0)

        def RETURN_TO(self):
            return self.getToken(XeasyPKParser.RETURN_TO, 0)

        def Float_TO(self, i:int=None):
            if i is None:
                return self.getTokens(XeasyPKParser.Float_TO)
            else:
                return self.getToken(XeasyPKParser.Float_TO, i)

        def getRuleIndex(self):
            return XeasyPKParser.RULE_tolerance

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterTolerance" ):
                listener.enterTolerance(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitTolerance" ):
                listener.exitTolerance(self)




    def tolerance(self):

        localctx = XeasyPKParser.ToleranceContext(self, self._ctx, self.state)
        self.enterRule(localctx, 12, self.RULE_tolerance)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 76
            self.match(XeasyPKParser.Tolerance)
            self.state = 78 
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while True:
                self.state = 77
                self.match(XeasyPKParser.Float_TO)
                self.state = 80 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if not (_la==35):
                    break

            self.state = 82
            self.match(XeasyPKParser.RETURN_TO)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Peak_list_2dContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def peak_2d(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(XeasyPKParser.Peak_2dContext)
            else:
                return self.getTypedRuleContext(XeasyPKParser.Peak_2dContext,i)


        def getRuleIndex(self):
            return XeasyPKParser.RULE_peak_list_2d

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPeak_list_2d" ):
                listener.enterPeak_list_2d(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPeak_list_2d" ):
                listener.exitPeak_list_2d(self)




    def peak_list_2d(self):

        localctx = XeasyPKParser.Peak_list_2dContext(self, self._ctx, self.state)
        self.enterRule(localctx, 14, self.RULE_peak_list_2d)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 85 
            self._errHandler.sync(self)
            _alt = 1
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt == 1:
                    self.state = 84
                    self.peak_2d()

                else:
                    raise NoViableAltException(self)
                self.state = 87 
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,5,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Peak_2dContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Integer(self, i:int=None):
            if i is None:
                return self.getTokens(XeasyPKParser.Integer)
            else:
                return self.getToken(XeasyPKParser.Integer, i)

        def Float(self, i:int=None):
            if i is None:
                return self.getTokens(XeasyPKParser.Float)
            else:
                return self.getToken(XeasyPKParser.Float, i)

        def Simple_name(self, i:int=None):
            if i is None:
                return self.getTokens(XeasyPKParser.Simple_name)
            else:
                return self.getToken(XeasyPKParser.Simple_name, i)

        def number(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(XeasyPKParser.NumberContext)
            else:
                return self.getTypedRuleContext(XeasyPKParser.NumberContext,i)


        def assign(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(XeasyPKParser.AssignContext)
            else:
                return self.getTypedRuleContext(XeasyPKParser.AssignContext,i)


        def RETURN(self, i:int=None):
            if i is None:
                return self.getTokens(XeasyPKParser.RETURN)
            else:
                return self.getToken(XeasyPKParser.RETURN, i)

        def comment(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(XeasyPKParser.CommentContext)
            else:
                return self.getTypedRuleContext(XeasyPKParser.CommentContext,i)


        def getRuleIndex(self):
            return XeasyPKParser.RULE_peak_2d

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPeak_2d" ):
                listener.enterPeak_2d(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPeak_2d" ):
                listener.exitPeak_2d(self)




    def peak_2d(self):

        localctx = XeasyPKParser.Peak_2dContext(self, self._ctx, self.state)
        self.enterRule(localctx, 16, self.RULE_peak_2d)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 89
            self.match(XeasyPKParser.Integer)
            self.state = 90
            self.match(XeasyPKParser.Float)
            self.state = 91
            self.match(XeasyPKParser.Float)
            self.state = 92
            self.match(XeasyPKParser.Integer)
            self.state = 93
            self.match(XeasyPKParser.Simple_name)
            self.state = 94
            self.number()
            self.state = 95
            self.number()
            self.state = 96
            self.match(XeasyPKParser.Simple_name)
            self.state = 97
            self.match(XeasyPKParser.Integer)
            self.state = 98
            self.assign()
            self.state = 99
            self.assign()
            self.state = 101
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==7:
                self.state = 100
                self.match(XeasyPKParser.Integer)


            self.state = 105
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [15]:
                self.state = 103
                self.match(XeasyPKParser.RETURN)
                pass
            elif token in [10]:
                self.state = 104
                self.comment()
                pass
            else:
                raise NoViableAltException(self)

            self.state = 118
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,10,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 107
                    self.assign()
                    self.state = 108
                    self.assign()
                    self.state = 110
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    if _la==7:
                        self.state = 109
                        self.match(XeasyPKParser.Integer)


                    self.state = 114
                    self._errHandler.sync(self)
                    token = self._input.LA(1)
                    if token in [15]:
                        self.state = 112
                        self.match(XeasyPKParser.RETURN)
                        pass
                    elif token in [10]:
                        self.state = 113
                        self.comment()
                        pass
                    else:
                        raise NoViableAltException(self)
             
                self.state = 120
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,10,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Peak_list_3dContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def peak_3d(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(XeasyPKParser.Peak_3dContext)
            else:
                return self.getTypedRuleContext(XeasyPKParser.Peak_3dContext,i)


        def getRuleIndex(self):
            return XeasyPKParser.RULE_peak_list_3d

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPeak_list_3d" ):
                listener.enterPeak_list_3d(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPeak_list_3d" ):
                listener.exitPeak_list_3d(self)




    def peak_list_3d(self):

        localctx = XeasyPKParser.Peak_list_3dContext(self, self._ctx, self.state)
        self.enterRule(localctx, 18, self.RULE_peak_list_3d)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 122 
            self._errHandler.sync(self)
            _alt = 1
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt == 1:
                    self.state = 121
                    self.peak_3d()

                else:
                    raise NoViableAltException(self)
                self.state = 124 
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,11,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Peak_3dContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Integer(self, i:int=None):
            if i is None:
                return self.getTokens(XeasyPKParser.Integer)
            else:
                return self.getToken(XeasyPKParser.Integer, i)

        def Float(self, i:int=None):
            if i is None:
                return self.getTokens(XeasyPKParser.Float)
            else:
                return self.getToken(XeasyPKParser.Float, i)

        def Simple_name(self, i:int=None):
            if i is None:
                return self.getTokens(XeasyPKParser.Simple_name)
            else:
                return self.getToken(XeasyPKParser.Simple_name, i)

        def number(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(XeasyPKParser.NumberContext)
            else:
                return self.getTypedRuleContext(XeasyPKParser.NumberContext,i)


        def assign(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(XeasyPKParser.AssignContext)
            else:
                return self.getTypedRuleContext(XeasyPKParser.AssignContext,i)


        def RETURN(self, i:int=None):
            if i is None:
                return self.getTokens(XeasyPKParser.RETURN)
            else:
                return self.getToken(XeasyPKParser.RETURN, i)

        def comment(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(XeasyPKParser.CommentContext)
            else:
                return self.getTypedRuleContext(XeasyPKParser.CommentContext,i)


        def getRuleIndex(self):
            return XeasyPKParser.RULE_peak_3d

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPeak_3d" ):
                listener.enterPeak_3d(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPeak_3d" ):
                listener.exitPeak_3d(self)




    def peak_3d(self):

        localctx = XeasyPKParser.Peak_3dContext(self, self._ctx, self.state)
        self.enterRule(localctx, 20, self.RULE_peak_3d)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 126
            self.match(XeasyPKParser.Integer)
            self.state = 127
            self.match(XeasyPKParser.Float)
            self.state = 128
            self.match(XeasyPKParser.Float)
            self.state = 129
            self.match(XeasyPKParser.Float)
            self.state = 130
            self.match(XeasyPKParser.Integer)
            self.state = 131
            self.match(XeasyPKParser.Simple_name)
            self.state = 132
            self.number()
            self.state = 133
            self.number()
            self.state = 134
            self.match(XeasyPKParser.Simple_name)
            self.state = 135
            self.match(XeasyPKParser.Integer)
            self.state = 136
            self.assign()
            self.state = 137
            self.assign()
            self.state = 138
            self.assign()
            self.state = 140
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==7:
                self.state = 139
                self.match(XeasyPKParser.Integer)


            self.state = 144
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [15]:
                self.state = 142
                self.match(XeasyPKParser.RETURN)
                pass
            elif token in [10]:
                self.state = 143
                self.comment()
                pass
            else:
                raise NoViableAltException(self)

            self.state = 158
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,16,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 146
                    self.assign()
                    self.state = 147
                    self.assign()
                    self.state = 148
                    self.assign()
                    self.state = 150
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    if _la==7:
                        self.state = 149
                        self.match(XeasyPKParser.Integer)


                    self.state = 154
                    self._errHandler.sync(self)
                    token = self._input.LA(1)
                    if token in [15]:
                        self.state = 152
                        self.match(XeasyPKParser.RETURN)
                        pass
                    elif token in [10]:
                        self.state = 153
                        self.comment()
                        pass
                    else:
                        raise NoViableAltException(self)
             
                self.state = 160
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,16,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Peak_list_4dContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def peak_4d(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(XeasyPKParser.Peak_4dContext)
            else:
                return self.getTypedRuleContext(XeasyPKParser.Peak_4dContext,i)


        def getRuleIndex(self):
            return XeasyPKParser.RULE_peak_list_4d

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPeak_list_4d" ):
                listener.enterPeak_list_4d(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPeak_list_4d" ):
                listener.exitPeak_list_4d(self)




    def peak_list_4d(self):

        localctx = XeasyPKParser.Peak_list_4dContext(self, self._ctx, self.state)
        self.enterRule(localctx, 22, self.RULE_peak_list_4d)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 162 
            self._errHandler.sync(self)
            _alt = 1
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt == 1:
                    self.state = 161
                    self.peak_4d()

                else:
                    raise NoViableAltException(self)
                self.state = 164 
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,17,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Peak_4dContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Integer(self, i:int=None):
            if i is None:
                return self.getTokens(XeasyPKParser.Integer)
            else:
                return self.getToken(XeasyPKParser.Integer, i)

        def Float(self, i:int=None):
            if i is None:
                return self.getTokens(XeasyPKParser.Float)
            else:
                return self.getToken(XeasyPKParser.Float, i)

        def Simple_name(self, i:int=None):
            if i is None:
                return self.getTokens(XeasyPKParser.Simple_name)
            else:
                return self.getToken(XeasyPKParser.Simple_name, i)

        def number(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(XeasyPKParser.NumberContext)
            else:
                return self.getTypedRuleContext(XeasyPKParser.NumberContext,i)


        def assign(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(XeasyPKParser.AssignContext)
            else:
                return self.getTypedRuleContext(XeasyPKParser.AssignContext,i)


        def RETURN(self, i:int=None):
            if i is None:
                return self.getTokens(XeasyPKParser.RETURN)
            else:
                return self.getToken(XeasyPKParser.RETURN, i)

        def comment(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(XeasyPKParser.CommentContext)
            else:
                return self.getTypedRuleContext(XeasyPKParser.CommentContext,i)


        def getRuleIndex(self):
            return XeasyPKParser.RULE_peak_4d

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPeak_4d" ):
                listener.enterPeak_4d(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPeak_4d" ):
                listener.exitPeak_4d(self)




    def peak_4d(self):

        localctx = XeasyPKParser.Peak_4dContext(self, self._ctx, self.state)
        self.enterRule(localctx, 24, self.RULE_peak_4d)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 166
            self.match(XeasyPKParser.Integer)
            self.state = 167
            self.match(XeasyPKParser.Float)
            self.state = 168
            self.match(XeasyPKParser.Float)
            self.state = 169
            self.match(XeasyPKParser.Float)
            self.state = 170
            self.match(XeasyPKParser.Float)
            self.state = 171
            self.match(XeasyPKParser.Integer)
            self.state = 172
            self.match(XeasyPKParser.Simple_name)
            self.state = 173
            self.number()
            self.state = 174
            self.number()
            self.state = 175
            self.match(XeasyPKParser.Simple_name)
            self.state = 176
            self.match(XeasyPKParser.Integer)
            self.state = 177
            self.assign()
            self.state = 178
            self.assign()
            self.state = 179
            self.assign()
            self.state = 180
            self.assign()
            self.state = 182
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==7:
                self.state = 181
                self.match(XeasyPKParser.Integer)


            self.state = 186
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [15]:
                self.state = 184
                self.match(XeasyPKParser.RETURN)
                pass
            elif token in [10]:
                self.state = 185
                self.comment()
                pass
            else:
                raise NoViableAltException(self)

            self.state = 201
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,22,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 188
                    self.assign()
                    self.state = 189
                    self.assign()
                    self.state = 190
                    self.assign()
                    self.state = 191
                    self.assign()
                    self.state = 193
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    if _la==7:
                        self.state = 192
                        self.match(XeasyPKParser.Integer)


                    self.state = 197
                    self._errHandler.sync(self)
                    token = self._input.LA(1)
                    if token in [15]:
                        self.state = 195
                        self.match(XeasyPKParser.RETURN)
                        pass
                    elif token in [10]:
                        self.state = 196
                        self.comment()
                        pass
                    else:
                        raise NoViableAltException(self)
             
                self.state = 203
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,22,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class NumberContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Float(self):
            return self.getToken(XeasyPKParser.Float, 0)

        def Real(self):
            return self.getToken(XeasyPKParser.Real, 0)

        def Integer(self):
            return self.getToken(XeasyPKParser.Integer, 0)

        def Simple_name(self):
            return self.getToken(XeasyPKParser.Simple_name, 0)

        def getRuleIndex(self):
            return XeasyPKParser.RULE_number

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterNumber" ):
                listener.enterNumber(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitNumber" ):
                listener.exitNumber(self)




    def number(self):

        localctx = XeasyPKParser.NumberContext(self, self._ctx, self.state)
        self.enterRule(localctx, 26, self.RULE_number)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 204
            _la = self._input.LA(1)
            if not((((_la) & ~0x3f) == 0 and ((1 << _la) & 9088) != 0)):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AssignContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Integer(self):
            return self.getToken(XeasyPKParser.Integer, 0)

        def Simple_name(self):
            return self.getToken(XeasyPKParser.Simple_name, 0)

        def getRuleIndex(self):
            return XeasyPKParser.RULE_assign

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAssign" ):
                listener.enterAssign(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAssign" ):
                listener.exitAssign(self)




    def assign(self):

        localctx = XeasyPKParser.AssignContext(self, self._ctx, self.state)
        self.enterRule(localctx, 28, self.RULE_assign)
        try:
            self.state = 211
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [7]:
                self.enterOuterAlt(localctx, 1)
                self.state = 206
                self.match(XeasyPKParser.Integer)
                pass
            elif token in [13]:
                self.enterOuterAlt(localctx, 2)
                self.state = 207
                self.match(XeasyPKParser.Simple_name)
                self.state = 209
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,23,self._ctx)
                if la_ == 1:
                    self.state = 208
                    self.match(XeasyPKParser.Integer)


                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class CommentContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def COMMENT(self):
            return self.getToken(XeasyPKParser.COMMENT, 0)

        def RETURN_CM(self):
            return self.getToken(XeasyPKParser.RETURN_CM, 0)

        def EOF(self):
            return self.getToken(XeasyPKParser.EOF, 0)

        def Any_name(self, i:int=None):
            if i is None:
                return self.getTokens(XeasyPKParser.Any_name)
            else:
                return self.getToken(XeasyPKParser.Any_name, i)

        def getRuleIndex(self):
            return XeasyPKParser.RULE_comment

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterComment" ):
                listener.enterComment(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitComment" ):
                listener.exitComment(self)




    def comment(self):

        localctx = XeasyPKParser.CommentContext(self, self._ctx, self.state)
        self.enterRule(localctx, 30, self.RULE_comment)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 213
            self.match(XeasyPKParser.COMMENT)
            self.state = 217
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==38:
                self.state = 214
                self.match(XeasyPKParser.Any_name)
                self.state = 219
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 220
            _la = self._input.LA(1)
            if not(_la==-1 or _la==40):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx








# Generated from CyanaNOAParser.g4 by ANTLR 4.13.0
# encoding: utf-8
from antlr4 import *
from io import StringIO
import sys
if sys.version_info[1] > 5:
	from typing import TextIO
else:
	from typing.io import TextIO

def serializedATN():
    return [
        4,1,71,204,2,0,7,0,2,1,7,1,2,2,7,2,2,3,7,3,2,4,7,4,2,5,7,5,2,6,7,
        6,2,7,7,7,2,8,7,8,2,9,7,9,2,10,7,10,2,11,7,11,1,0,1,0,1,0,1,0,5,
        0,29,8,0,10,0,12,0,32,9,0,1,0,1,0,1,1,1,1,5,1,38,8,1,10,1,12,1,41,
        9,1,1,1,1,1,1,2,1,2,1,2,3,2,48,8,2,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,
        3,1,3,1,3,1,3,1,3,1,3,1,3,3,3,64,8,3,1,3,1,3,1,3,1,4,1,4,1,4,1,4,
        1,4,1,4,1,4,1,4,1,4,1,4,1,5,4,5,80,8,5,11,5,12,5,81,1,5,1,5,1,5,
        1,5,1,5,3,5,89,8,5,1,6,1,6,1,6,1,6,1,6,1,6,1,6,1,6,1,6,1,6,1,7,1,
        7,1,7,1,7,1,7,3,7,106,8,7,1,7,3,7,109,8,7,1,8,1,8,3,8,113,8,8,1,
        9,1,9,1,9,1,9,1,9,1,9,1,9,1,9,1,9,1,9,1,9,1,9,1,9,1,9,1,9,1,9,1,
        9,1,9,1,9,1,9,1,9,1,9,1,9,1,9,5,9,139,8,9,10,9,12,9,142,9,9,1,10,
        1,10,1,10,1,10,1,10,1,10,1,10,1,10,1,11,1,11,1,11,1,11,1,11,1,11,
        1,11,1,11,1,11,1,11,1,11,1,11,1,11,1,11,1,11,1,11,1,11,1,11,1,11,
        1,11,1,11,1,11,1,11,1,11,1,11,1,11,1,11,1,11,1,11,1,11,1,11,1,11,
        1,11,1,11,1,11,1,11,1,11,1,11,1,11,1,11,1,11,1,11,1,11,1,11,1,11,
        1,11,1,11,1,11,1,11,1,11,1,11,1,11,1,11,0,0,12,0,2,4,6,8,10,12,14,
        16,18,20,22,0,6,1,1,71,71,1,0,4,5,1,0,53,54,1,0,10,13,2,0,54,54,
        57,57,1,0,59,62,205,0,30,1,0,0,0,2,35,1,0,0,0,4,44,1,0,0,0,6,49,
        1,0,0,0,8,68,1,0,0,0,10,79,1,0,0,0,12,90,1,0,0,0,14,100,1,0,0,0,
        16,110,1,0,0,0,18,114,1,0,0,0,20,143,1,0,0,0,22,151,1,0,0,0,24,29,
        3,2,1,0,25,29,3,4,2,0,26,29,3,18,9,0,27,29,3,22,11,0,28,24,1,0,0,
        0,28,25,1,0,0,0,28,26,1,0,0,0,28,27,1,0,0,0,29,32,1,0,0,0,30,28,
        1,0,0,0,30,31,1,0,0,0,31,33,1,0,0,0,32,30,1,0,0,0,33,34,5,0,0,1,
        34,1,1,0,0,0,35,39,5,63,0,0,36,38,5,69,0,0,37,36,1,0,0,0,38,41,1,
        0,0,0,39,37,1,0,0,0,39,40,1,0,0,0,40,42,1,0,0,0,41,39,1,0,0,0,42,
        43,7,0,0,0,43,3,1,0,0,0,44,45,3,6,3,0,45,47,3,8,4,0,46,48,3,10,5,
        0,47,46,1,0,0,0,47,48,1,0,0,0,48,5,1,0,0,0,49,50,5,1,0,0,50,51,5,
        57,0,0,51,52,5,2,0,0,52,53,5,64,0,0,53,54,5,47,0,0,54,55,5,58,0,
        0,55,56,5,51,0,0,56,57,5,58,0,0,57,63,5,3,0,0,58,64,5,56,0,0,59,
        60,5,56,0,0,60,61,7,1,0,0,61,64,5,56,0,0,62,64,5,6,0,0,63,58,1,0,
        0,0,63,59,1,0,0,0,63,62,1,0,0,0,64,65,1,0,0,0,65,66,5,48,0,0,66,
        67,5,49,0,0,67,7,1,0,0,0,68,69,5,57,0,0,69,70,5,7,0,0,70,71,5,57,
        0,0,71,72,5,8,0,0,72,73,5,51,0,0,73,74,5,9,0,0,74,75,5,52,0,0,75,
        76,5,58,0,0,76,77,5,49,0,0,77,9,1,0,0,0,78,80,3,12,6,0,79,78,1,0,
        0,0,80,81,1,0,0,0,81,79,1,0,0,0,81,82,1,0,0,0,82,88,1,0,0,0,83,84,
        5,15,0,0,84,85,5,57,0,0,85,86,5,16,0,0,86,87,5,56,0,0,87,89,5,50,
        0,0,88,83,1,0,0,0,88,89,1,0,0,0,89,11,1,0,0,0,90,91,5,64,0,0,91,
        92,5,64,0,0,92,93,5,57,0,0,93,94,7,2,0,0,94,95,5,64,0,0,95,96,5,
        64,0,0,96,97,5,57,0,0,97,98,7,3,0,0,98,99,3,14,7,0,99,13,1,0,0,0,
        100,101,5,57,0,0,101,102,5,57,0,0,102,103,7,4,0,0,103,105,7,4,0,
        0,104,106,5,14,0,0,105,104,1,0,0,0,105,106,1,0,0,0,106,108,1,0,0,
        0,107,109,3,16,8,0,108,107,1,0,0,0,108,109,1,0,0,0,109,15,1,0,0,
        0,110,112,7,5,0,0,111,113,3,16,8,0,112,111,1,0,0,0,112,113,1,0,0,
        0,113,17,1,0,0,0,114,115,5,17,0,0,115,116,5,49,0,0,116,117,5,58,
        0,0,117,118,5,18,0,0,118,119,5,49,0,0,119,120,5,58,0,0,120,121,5,
        19,0,0,121,122,5,49,0,0,122,123,5,57,0,0,123,124,5,20,0,0,124,125,
        5,49,0,0,125,126,5,57,0,0,126,127,5,21,0,0,127,128,5,49,0,0,128,
        129,5,22,0,0,129,130,5,49,0,0,130,131,5,56,0,0,131,132,5,23,0,0,
        132,133,5,24,0,0,133,134,5,26,0,0,134,135,5,25,0,0,135,136,5,27,
        0,0,136,140,5,28,0,0,137,139,3,20,10,0,138,137,1,0,0,0,139,142,1,
        0,0,0,140,138,1,0,0,0,140,141,1,0,0,0,141,19,1,0,0,0,142,140,1,0,
        0,0,143,144,5,64,0,0,144,145,5,64,0,0,145,146,5,57,0,0,146,147,5,
        58,0,0,147,148,5,57,0,0,148,149,5,57,0,0,149,150,5,57,0,0,150,21,
        1,0,0,0,151,152,5,25,0,0,152,153,5,49,0,0,153,154,5,29,0,0,154,155,
        5,49,0,0,155,156,5,57,0,0,156,157,5,30,0,0,157,158,5,49,0,0,158,
        159,5,57,0,0,159,160,5,31,0,0,160,161,5,49,0,0,161,162,5,57,0,0,
        162,163,5,32,0,0,163,164,5,49,0,0,164,165,5,57,0,0,165,166,5,33,
        0,0,166,167,5,56,0,0,167,168,5,49,0,0,168,169,5,57,0,0,169,170,5,
        34,0,0,170,171,5,58,0,0,171,172,5,37,0,0,172,173,5,56,0,0,173,174,
        5,49,0,0,174,175,5,57,0,0,175,176,5,35,0,0,176,177,5,56,0,0,177,
        178,5,49,0,0,178,179,5,57,0,0,179,180,5,36,0,0,180,181,5,49,0,0,
        181,182,5,57,0,0,182,183,5,38,0,0,183,184,5,49,0,0,184,185,5,39,
        0,0,185,186,5,49,0,0,186,187,5,57,0,0,187,188,5,40,0,0,188,189,5,
        49,0,0,189,190,5,57,0,0,190,191,5,41,0,0,191,192,5,44,0,0,192,193,
        5,49,0,0,193,194,5,57,0,0,194,195,5,42,0,0,195,196,5,45,0,0,196,
        197,5,49,0,0,197,198,5,57,0,0,198,199,5,43,0,0,199,200,5,46,0,0,
        200,201,5,49,0,0,201,202,5,57,0,0,202,23,1,0,0,0,11,28,30,39,47,
        63,81,88,105,108,112,140
    ]

class CyanaNOAParser ( Parser ):

    grammarFileName = "CyanaNOAParser.g4"

    atn = ATNDeserializer().deserialize(serializedATN())

    decisionsToDFA = [ DFA(ds, i) for i, ds in enumerate(atn.decisionToState) ]

    sharedContextCache = PredictionContextCache()

    literalNames = [ "<INVALID>", "'|i-j|'", "'from'", "'ppm;'", "'increased from'", 
                     "'|i-j|'", "'diagonal'", "'out of'", "<INVALID>", "'quality'", 
                     "'OK'", "'lone'", "'poor'", "'far'", "<INVALID>", "'Violated in'", 
                     "'structures by'", "'Average quality of peak assignments'", 
                     "'Average number of used assignments'", "'Peaks with increased upper limit'", 
                     "'Peaks with decreased upper limit'", "'Protons used in less than 30% of expected peaks'", 
                     "'Peak observation distance'", "'Atom'", "'Residue'", 
                     "'Peaks'", "'Shift'", "'Used'", "'Expect'", "'selected'", 
                     "'assigned'", "'unassigned'", "'without assignment possibility'", 
                     "'with violation below'", "'with violation between'", 
                     "'with violation above'", "'with diagonal assignment'", 
                     "'and'", "'Cross peaks'", "'with off-diagonal assignment'", 
                     "'with unique assignment'", "'with short-range assignment'", 
                     "'with medium-range assignment'", "'with long-range assignment'", 
                     "<INVALID>", "'1<|i-j|<5'", "<INVALID>", "'('", "')'", 
                     "':'", "'.'", "','", "'='", "'+'", "'-'", "'/'" ]

    symbolicNames = [ "<INVALID>", "Peak", "From", "Ppm_SC", "Increased_from", 
                      "Decreased_from", "Diagonal", "Out_of", "Assignments_used", 
                      "Quality", "Ok", "Lone", "Poor", "Far", "Distance_range", 
                      "Violated_in", "Structures_by", "Average_quality", 
                      "Average_number", "Peaks_inc_upl", "Peaks_dec_upl", 
                      "Protons_used_in_less", "Peak_obs_dist", "Atom", "Residue", 
                      "Peaks", "Shift", "Used", "Expect", "Selected", "Assigned", 
                      "Unassigned", "Without_possibility", "With_viol_below", 
                      "With_viol_between", "With_viol_above", "With_diagonal", 
                      "And", "Cross_peaks", "With_off_diagonal", "With_unique", 
                      "With_short_range", "With_medium_range", "With_long_range", 
                      "Short_range_ex", "Medium_range_ex", "Long_range_ex", 
                      "L_paren", "R_paren", "Colon", "Period", "Comma", 
                      "Equ_op", "Add_op", "Sub_op", "Div_op", "Angstrome", 
                      "Integer", "Float", "Numerical_report1", "Numerical_report2", 
                      "Numerical_report3", "Numerical_report4", "COMMENT", 
                      "Simple_name", "SPACE", "ENCLOSE_COMMENT", "SECTION_COMMENT", 
                      "LINE_COMMENT", "Any_name", "SPACE_CM", "RETURN_CM" ]

    RULE_cyana_noa = 0
    RULE_comment = 1
    RULE_noe_peaks = 2
    RULE_peak_header = 3
    RULE_peak_quality = 4
    RULE_noe_assignments = 5
    RULE_noe_assignment = 6
    RULE_numerical_report = 7
    RULE_extended_report = 8
    RULE_noe_stat = 9
    RULE_list_of_proton = 10
    RULE_peak_stat = 11

    ruleNames =  [ "cyana_noa", "comment", "noe_peaks", "peak_header", "peak_quality", 
                   "noe_assignments", "noe_assignment", "numerical_report", 
                   "extended_report", "noe_stat", "list_of_proton", "peak_stat" ]

    EOF = Token.EOF
    Peak=1
    From=2
    Ppm_SC=3
    Increased_from=4
    Decreased_from=5
    Diagonal=6
    Out_of=7
    Assignments_used=8
    Quality=9
    Ok=10
    Lone=11
    Poor=12
    Far=13
    Distance_range=14
    Violated_in=15
    Structures_by=16
    Average_quality=17
    Average_number=18
    Peaks_inc_upl=19
    Peaks_dec_upl=20
    Protons_used_in_less=21
    Peak_obs_dist=22
    Atom=23
    Residue=24
    Peaks=25
    Shift=26
    Used=27
    Expect=28
    Selected=29
    Assigned=30
    Unassigned=31
    Without_possibility=32
    With_viol_below=33
    With_viol_between=34
    With_viol_above=35
    With_diagonal=36
    And=37
    Cross_peaks=38
    With_off_diagonal=39
    With_unique=40
    With_short_range=41
    With_medium_range=42
    With_long_range=43
    Short_range_ex=44
    Medium_range_ex=45
    Long_range_ex=46
    L_paren=47
    R_paren=48
    Colon=49
    Period=50
    Comma=51
    Equ_op=52
    Add_op=53
    Sub_op=54
    Div_op=55
    Angstrome=56
    Integer=57
    Float=58
    Numerical_report1=59
    Numerical_report2=60
    Numerical_report3=61
    Numerical_report4=62
    COMMENT=63
    Simple_name=64
    SPACE=65
    ENCLOSE_COMMENT=66
    SECTION_COMMENT=67
    LINE_COMMENT=68
    Any_name=69
    SPACE_CM=70
    RETURN_CM=71

    def __init__(self, input:TokenStream, output:TextIO = sys.stdout):
        super().__init__(input, output)
        self.checkVersion("4.13.0")
        self._interp = ParserATNSimulator(self, self.atn, self.decisionsToDFA, self.sharedContextCache)
        self._predicates = None




    class Cyana_noaContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def EOF(self):
            return self.getToken(CyanaNOAParser.EOF, 0)

        def comment(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CyanaNOAParser.CommentContext)
            else:
                return self.getTypedRuleContext(CyanaNOAParser.CommentContext,i)


        def noe_peaks(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CyanaNOAParser.Noe_peaksContext)
            else:
                return self.getTypedRuleContext(CyanaNOAParser.Noe_peaksContext,i)


        def noe_stat(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CyanaNOAParser.Noe_statContext)
            else:
                return self.getTypedRuleContext(CyanaNOAParser.Noe_statContext,i)


        def peak_stat(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CyanaNOAParser.Peak_statContext)
            else:
                return self.getTypedRuleContext(CyanaNOAParser.Peak_statContext,i)


        def getRuleIndex(self):
            return CyanaNOAParser.RULE_cyana_noa

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterCyana_noa" ):
                listener.enterCyana_noa(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitCyana_noa" ):
                listener.exitCyana_noa(self)




    def cyana_noa(self):

        localctx = CyanaNOAParser.Cyana_noaContext(self, self._ctx, self.state)
        self.enterRule(localctx, 0, self.RULE_cyana_noa)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 30
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while (((_la) & ~0x3f) == 0 and ((1 << _la) & -9223372036821090302) != 0):
                self.state = 28
                self._errHandler.sync(self)
                token = self._input.LA(1)
                if token in [63]:
                    self.state = 24
                    self.comment()
                    pass
                elif token in [1]:
                    self.state = 25
                    self.noe_peaks()
                    pass
                elif token in [17]:
                    self.state = 26
                    self.noe_stat()
                    pass
                elif token in [25]:
                    self.state = 27
                    self.peak_stat()
                    pass
                else:
                    raise NoViableAltException(self)

                self.state = 32
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 33
            self.match(CyanaNOAParser.EOF)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class CommentContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def COMMENT(self):
            return self.getToken(CyanaNOAParser.COMMENT, 0)

        def RETURN_CM(self):
            return self.getToken(CyanaNOAParser.RETURN_CM, 0)

        def EOF(self):
            return self.getToken(CyanaNOAParser.EOF, 0)

        def Any_name(self, i:int=None):
            if i is None:
                return self.getTokens(CyanaNOAParser.Any_name)
            else:
                return self.getToken(CyanaNOAParser.Any_name, i)

        def getRuleIndex(self):
            return CyanaNOAParser.RULE_comment

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterComment" ):
                listener.enterComment(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitComment" ):
                listener.exitComment(self)




    def comment(self):

        localctx = CyanaNOAParser.CommentContext(self, self._ctx, self.state)
        self.enterRule(localctx, 2, self.RULE_comment)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 35
            self.match(CyanaNOAParser.COMMENT)
            self.state = 39
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==69:
                self.state = 36
                self.match(CyanaNOAParser.Any_name)
                self.state = 41
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 42
            _la = self._input.LA(1)
            if not(_la==-1 or _la==71):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Noe_peaksContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def peak_header(self):
            return self.getTypedRuleContext(CyanaNOAParser.Peak_headerContext,0)


        def peak_quality(self):
            return self.getTypedRuleContext(CyanaNOAParser.Peak_qualityContext,0)


        def noe_assignments(self):
            return self.getTypedRuleContext(CyanaNOAParser.Noe_assignmentsContext,0)


        def getRuleIndex(self):
            return CyanaNOAParser.RULE_noe_peaks

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterNoe_peaks" ):
                listener.enterNoe_peaks(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitNoe_peaks" ):
                listener.exitNoe_peaks(self)




    def noe_peaks(self):

        localctx = CyanaNOAParser.Noe_peaksContext(self, self._ctx, self.state)
        self.enterRule(localctx, 4, self.RULE_noe_peaks)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 44
            self.peak_header()
            self.state = 45
            self.peak_quality()
            self.state = 47
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==64:
                self.state = 46
                self.noe_assignments()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Peak_headerContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Peak(self):
            return self.getToken(CyanaNOAParser.Peak, 0)

        def Integer(self):
            return self.getToken(CyanaNOAParser.Integer, 0)

        def From(self):
            return self.getToken(CyanaNOAParser.From, 0)

        def Simple_name(self):
            return self.getToken(CyanaNOAParser.Simple_name, 0)

        def L_paren(self):
            return self.getToken(CyanaNOAParser.L_paren, 0)

        def Float(self, i:int=None):
            if i is None:
                return self.getTokens(CyanaNOAParser.Float)
            else:
                return self.getToken(CyanaNOAParser.Float, i)

        def Comma(self):
            return self.getToken(CyanaNOAParser.Comma, 0)

        def Ppm_SC(self):
            return self.getToken(CyanaNOAParser.Ppm_SC, 0)

        def R_paren(self):
            return self.getToken(CyanaNOAParser.R_paren, 0)

        def Colon(self):
            return self.getToken(CyanaNOAParser.Colon, 0)

        def Angstrome(self, i:int=None):
            if i is None:
                return self.getTokens(CyanaNOAParser.Angstrome)
            else:
                return self.getToken(CyanaNOAParser.Angstrome, i)

        def Diagonal(self):
            return self.getToken(CyanaNOAParser.Diagonal, 0)

        def Increased_from(self):
            return self.getToken(CyanaNOAParser.Increased_from, 0)

        def Decreased_from(self):
            return self.getToken(CyanaNOAParser.Decreased_from, 0)

        def getRuleIndex(self):
            return CyanaNOAParser.RULE_peak_header

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPeak_header" ):
                listener.enterPeak_header(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPeak_header" ):
                listener.exitPeak_header(self)




    def peak_header(self):

        localctx = CyanaNOAParser.Peak_headerContext(self, self._ctx, self.state)
        self.enterRule(localctx, 6, self.RULE_peak_header)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 49
            self.match(CyanaNOAParser.Peak)
            self.state = 50
            self.match(CyanaNOAParser.Integer)
            self.state = 51
            self.match(CyanaNOAParser.From)
            self.state = 52
            self.match(CyanaNOAParser.Simple_name)
            self.state = 53
            self.match(CyanaNOAParser.L_paren)
            self.state = 54
            self.match(CyanaNOAParser.Float)
            self.state = 55
            self.match(CyanaNOAParser.Comma)
            self.state = 56
            self.match(CyanaNOAParser.Float)
            self.state = 57
            self.match(CyanaNOAParser.Ppm_SC)
            self.state = 63
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,4,self._ctx)
            if la_ == 1:
                self.state = 58
                self.match(CyanaNOAParser.Angstrome)
                pass

            elif la_ == 2:
                self.state = 59
                self.match(CyanaNOAParser.Angstrome)
                self.state = 60
                _la = self._input.LA(1)
                if not(_la==4 or _la==5):
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()
                self.state = 61
                self.match(CyanaNOAParser.Angstrome)
                pass

            elif la_ == 3:
                self.state = 62
                self.match(CyanaNOAParser.Diagonal)
                pass


            self.state = 65
            self.match(CyanaNOAParser.R_paren)
            self.state = 66
            self.match(CyanaNOAParser.Colon)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Peak_qualityContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Integer(self, i:int=None):
            if i is None:
                return self.getTokens(CyanaNOAParser.Integer)
            else:
                return self.getToken(CyanaNOAParser.Integer, i)

        def Out_of(self):
            return self.getToken(CyanaNOAParser.Out_of, 0)

        def Assignments_used(self):
            return self.getToken(CyanaNOAParser.Assignments_used, 0)

        def Comma(self):
            return self.getToken(CyanaNOAParser.Comma, 0)

        def Quality(self):
            return self.getToken(CyanaNOAParser.Quality, 0)

        def Equ_op(self):
            return self.getToken(CyanaNOAParser.Equ_op, 0)

        def Float(self):
            return self.getToken(CyanaNOAParser.Float, 0)

        def Colon(self):
            return self.getToken(CyanaNOAParser.Colon, 0)

        def getRuleIndex(self):
            return CyanaNOAParser.RULE_peak_quality

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPeak_quality" ):
                listener.enterPeak_quality(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPeak_quality" ):
                listener.exitPeak_quality(self)




    def peak_quality(self):

        localctx = CyanaNOAParser.Peak_qualityContext(self, self._ctx, self.state)
        self.enterRule(localctx, 8, self.RULE_peak_quality)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 68
            self.match(CyanaNOAParser.Integer)
            self.state = 69
            self.match(CyanaNOAParser.Out_of)
            self.state = 70
            self.match(CyanaNOAParser.Integer)
            self.state = 71
            self.match(CyanaNOAParser.Assignments_used)
            self.state = 72
            self.match(CyanaNOAParser.Comma)
            self.state = 73
            self.match(CyanaNOAParser.Quality)
            self.state = 74
            self.match(CyanaNOAParser.Equ_op)
            self.state = 75
            self.match(CyanaNOAParser.Float)
            self.state = 76
            self.match(CyanaNOAParser.Colon)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Noe_assignmentsContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def noe_assignment(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CyanaNOAParser.Noe_assignmentContext)
            else:
                return self.getTypedRuleContext(CyanaNOAParser.Noe_assignmentContext,i)


        def Violated_in(self):
            return self.getToken(CyanaNOAParser.Violated_in, 0)

        def Integer(self):
            return self.getToken(CyanaNOAParser.Integer, 0)

        def Structures_by(self):
            return self.getToken(CyanaNOAParser.Structures_by, 0)

        def Angstrome(self):
            return self.getToken(CyanaNOAParser.Angstrome, 0)

        def Period(self):
            return self.getToken(CyanaNOAParser.Period, 0)

        def getRuleIndex(self):
            return CyanaNOAParser.RULE_noe_assignments

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterNoe_assignments" ):
                listener.enterNoe_assignments(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitNoe_assignments" ):
                listener.exitNoe_assignments(self)




    def noe_assignments(self):

        localctx = CyanaNOAParser.Noe_assignmentsContext(self, self._ctx, self.state)
        self.enterRule(localctx, 10, self.RULE_noe_assignments)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 79 
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while True:
                self.state = 78
                self.noe_assignment()
                self.state = 81 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if not (_la==64):
                    break

            self.state = 88
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==15:
                self.state = 83
                self.match(CyanaNOAParser.Violated_in)
                self.state = 84
                self.match(CyanaNOAParser.Integer)
                self.state = 85
                self.match(CyanaNOAParser.Structures_by)
                self.state = 86
                self.match(CyanaNOAParser.Angstrome)
                self.state = 87
                self.match(CyanaNOAParser.Period)


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Noe_assignmentContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Simple_name(self, i:int=None):
            if i is None:
                return self.getTokens(CyanaNOAParser.Simple_name)
            else:
                return self.getToken(CyanaNOAParser.Simple_name, i)

        def Integer(self, i:int=None):
            if i is None:
                return self.getTokens(CyanaNOAParser.Integer)
            else:
                return self.getToken(CyanaNOAParser.Integer, i)

        def numerical_report(self):
            return self.getTypedRuleContext(CyanaNOAParser.Numerical_reportContext,0)


        def Add_op(self):
            return self.getToken(CyanaNOAParser.Add_op, 0)

        def Sub_op(self):
            return self.getToken(CyanaNOAParser.Sub_op, 0)

        def Ok(self):
            return self.getToken(CyanaNOAParser.Ok, 0)

        def Lone(self):
            return self.getToken(CyanaNOAParser.Lone, 0)

        def Poor(self):
            return self.getToken(CyanaNOAParser.Poor, 0)

        def Far(self):
            return self.getToken(CyanaNOAParser.Far, 0)

        def getRuleIndex(self):
            return CyanaNOAParser.RULE_noe_assignment

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterNoe_assignment" ):
                listener.enterNoe_assignment(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitNoe_assignment" ):
                listener.exitNoe_assignment(self)




    def noe_assignment(self):

        localctx = CyanaNOAParser.Noe_assignmentContext(self, self._ctx, self.state)
        self.enterRule(localctx, 12, self.RULE_noe_assignment)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 90
            self.match(CyanaNOAParser.Simple_name)
            self.state = 91
            self.match(CyanaNOAParser.Simple_name)
            self.state = 92
            self.match(CyanaNOAParser.Integer)
            self.state = 93
            _la = self._input.LA(1)
            if not(_la==53 or _la==54):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
            self.state = 94
            self.match(CyanaNOAParser.Simple_name)
            self.state = 95
            self.match(CyanaNOAParser.Simple_name)
            self.state = 96
            self.match(CyanaNOAParser.Integer)
            self.state = 97
            _la = self._input.LA(1)
            if not((((_la) & ~0x3f) == 0 and ((1 << _la) & 15360) != 0)):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
            self.state = 98
            self.numerical_report()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Numerical_reportContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Integer(self, i:int=None):
            if i is None:
                return self.getTokens(CyanaNOAParser.Integer)
            else:
                return self.getToken(CyanaNOAParser.Integer, i)

        def Sub_op(self, i:int=None):
            if i is None:
                return self.getTokens(CyanaNOAParser.Sub_op)
            else:
                return self.getToken(CyanaNOAParser.Sub_op, i)

        def Distance_range(self):
            return self.getToken(CyanaNOAParser.Distance_range, 0)

        def extended_report(self):
            return self.getTypedRuleContext(CyanaNOAParser.Extended_reportContext,0)


        def getRuleIndex(self):
            return CyanaNOAParser.RULE_numerical_report

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterNumerical_report" ):
                listener.enterNumerical_report(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitNumerical_report" ):
                listener.exitNumerical_report(self)




    def numerical_report(self):

        localctx = CyanaNOAParser.Numerical_reportContext(self, self._ctx, self.state)
        self.enterRule(localctx, 14, self.RULE_numerical_report)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 100
            self.match(CyanaNOAParser.Integer)
            self.state = 101
            self.match(CyanaNOAParser.Integer)
            self.state = 102
            _la = self._input.LA(1)
            if not(_la==54 or _la==57):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
            self.state = 103
            _la = self._input.LA(1)
            if not(_la==54 or _la==57):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
            self.state = 105
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==14:
                self.state = 104
                self.match(CyanaNOAParser.Distance_range)


            self.state = 108
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if (((_la) & ~0x3f) == 0 and ((1 << _la) & 8646911284551352320) != 0):
                self.state = 107
                self.extended_report()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Extended_reportContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Numerical_report1(self):
            return self.getToken(CyanaNOAParser.Numerical_report1, 0)

        def Numerical_report2(self):
            return self.getToken(CyanaNOAParser.Numerical_report2, 0)

        def Numerical_report3(self):
            return self.getToken(CyanaNOAParser.Numerical_report3, 0)

        def Numerical_report4(self):
            return self.getToken(CyanaNOAParser.Numerical_report4, 0)

        def extended_report(self):
            return self.getTypedRuleContext(CyanaNOAParser.Extended_reportContext,0)


        def getRuleIndex(self):
            return CyanaNOAParser.RULE_extended_report

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterExtended_report" ):
                listener.enterExtended_report(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitExtended_report" ):
                listener.exitExtended_report(self)




    def extended_report(self):

        localctx = CyanaNOAParser.Extended_reportContext(self, self._ctx, self.state)
        self.enterRule(localctx, 16, self.RULE_extended_report)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 110
            _la = self._input.LA(1)
            if not((((_la) & ~0x3f) == 0 and ((1 << _la) & 8646911284551352320) != 0)):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
            self.state = 112
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if (((_la) & ~0x3f) == 0 and ((1 << _la) & 8646911284551352320) != 0):
                self.state = 111
                self.extended_report()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Noe_statContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Average_quality(self):
            return self.getToken(CyanaNOAParser.Average_quality, 0)

        def Colon(self, i:int=None):
            if i is None:
                return self.getTokens(CyanaNOAParser.Colon)
            else:
                return self.getToken(CyanaNOAParser.Colon, i)

        def Float(self, i:int=None):
            if i is None:
                return self.getTokens(CyanaNOAParser.Float)
            else:
                return self.getToken(CyanaNOAParser.Float, i)

        def Average_number(self):
            return self.getToken(CyanaNOAParser.Average_number, 0)

        def Peaks_inc_upl(self):
            return self.getToken(CyanaNOAParser.Peaks_inc_upl, 0)

        def Integer(self, i:int=None):
            if i is None:
                return self.getTokens(CyanaNOAParser.Integer)
            else:
                return self.getToken(CyanaNOAParser.Integer, i)

        def Peaks_dec_upl(self):
            return self.getToken(CyanaNOAParser.Peaks_dec_upl, 0)

        def Protons_used_in_less(self):
            return self.getToken(CyanaNOAParser.Protons_used_in_less, 0)

        def Peak_obs_dist(self):
            return self.getToken(CyanaNOAParser.Peak_obs_dist, 0)

        def Angstrome(self):
            return self.getToken(CyanaNOAParser.Angstrome, 0)

        def Atom(self):
            return self.getToken(CyanaNOAParser.Atom, 0)

        def Residue(self):
            return self.getToken(CyanaNOAParser.Residue, 0)

        def Shift(self):
            return self.getToken(CyanaNOAParser.Shift, 0)

        def Peaks(self):
            return self.getToken(CyanaNOAParser.Peaks, 0)

        def Used(self):
            return self.getToken(CyanaNOAParser.Used, 0)

        def Expect(self):
            return self.getToken(CyanaNOAParser.Expect, 0)

        def list_of_proton(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CyanaNOAParser.List_of_protonContext)
            else:
                return self.getTypedRuleContext(CyanaNOAParser.List_of_protonContext,i)


        def getRuleIndex(self):
            return CyanaNOAParser.RULE_noe_stat

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterNoe_stat" ):
                listener.enterNoe_stat(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitNoe_stat" ):
                listener.exitNoe_stat(self)




    def noe_stat(self):

        localctx = CyanaNOAParser.Noe_statContext(self, self._ctx, self.state)
        self.enterRule(localctx, 18, self.RULE_noe_stat)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 114
            self.match(CyanaNOAParser.Average_quality)
            self.state = 115
            self.match(CyanaNOAParser.Colon)
            self.state = 116
            self.match(CyanaNOAParser.Float)
            self.state = 117
            self.match(CyanaNOAParser.Average_number)
            self.state = 118
            self.match(CyanaNOAParser.Colon)
            self.state = 119
            self.match(CyanaNOAParser.Float)
            self.state = 120
            self.match(CyanaNOAParser.Peaks_inc_upl)
            self.state = 121
            self.match(CyanaNOAParser.Colon)
            self.state = 122
            self.match(CyanaNOAParser.Integer)
            self.state = 123
            self.match(CyanaNOAParser.Peaks_dec_upl)
            self.state = 124
            self.match(CyanaNOAParser.Colon)
            self.state = 125
            self.match(CyanaNOAParser.Integer)
            self.state = 126
            self.match(CyanaNOAParser.Protons_used_in_less)
            self.state = 127
            self.match(CyanaNOAParser.Colon)
            self.state = 128
            self.match(CyanaNOAParser.Peak_obs_dist)
            self.state = 129
            self.match(CyanaNOAParser.Colon)
            self.state = 130
            self.match(CyanaNOAParser.Angstrome)
            self.state = 131
            self.match(CyanaNOAParser.Atom)
            self.state = 132
            self.match(CyanaNOAParser.Residue)
            self.state = 133
            self.match(CyanaNOAParser.Shift)
            self.state = 134
            self.match(CyanaNOAParser.Peaks)
            self.state = 135
            self.match(CyanaNOAParser.Used)
            self.state = 136
            self.match(CyanaNOAParser.Expect)
            self.state = 140
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==64:
                self.state = 137
                self.list_of_proton()
                self.state = 142
                self._errHandler.sync(self)
                _la = self._input.LA(1)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class List_of_protonContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Simple_name(self, i:int=None):
            if i is None:
                return self.getTokens(CyanaNOAParser.Simple_name)
            else:
                return self.getToken(CyanaNOAParser.Simple_name, i)

        def Integer(self, i:int=None):
            if i is None:
                return self.getTokens(CyanaNOAParser.Integer)
            else:
                return self.getToken(CyanaNOAParser.Integer, i)

        def Float(self):
            return self.getToken(CyanaNOAParser.Float, 0)

        def getRuleIndex(self):
            return CyanaNOAParser.RULE_list_of_proton

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterList_of_proton" ):
                listener.enterList_of_proton(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitList_of_proton" ):
                listener.exitList_of_proton(self)




    def list_of_proton(self):

        localctx = CyanaNOAParser.List_of_protonContext(self, self._ctx, self.state)
        self.enterRule(localctx, 20, self.RULE_list_of_proton)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 143
            self.match(CyanaNOAParser.Simple_name)
            self.state = 144
            self.match(CyanaNOAParser.Simple_name)
            self.state = 145
            self.match(CyanaNOAParser.Integer)
            self.state = 146
            self.match(CyanaNOAParser.Float)
            self.state = 147
            self.match(CyanaNOAParser.Integer)
            self.state = 148
            self.match(CyanaNOAParser.Integer)
            self.state = 149
            self.match(CyanaNOAParser.Integer)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Peak_statContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Peaks(self):
            return self.getToken(CyanaNOAParser.Peaks, 0)

        def Colon(self, i:int=None):
            if i is None:
                return self.getTokens(CyanaNOAParser.Colon)
            else:
                return self.getToken(CyanaNOAParser.Colon, i)

        def Selected(self):
            return self.getToken(CyanaNOAParser.Selected, 0)

        def Integer(self, i:int=None):
            if i is None:
                return self.getTokens(CyanaNOAParser.Integer)
            else:
                return self.getToken(CyanaNOAParser.Integer, i)

        def Assigned(self):
            return self.getToken(CyanaNOAParser.Assigned, 0)

        def Unassigned(self):
            return self.getToken(CyanaNOAParser.Unassigned, 0)

        def Without_possibility(self):
            return self.getToken(CyanaNOAParser.Without_possibility, 0)

        def With_viol_below(self):
            return self.getToken(CyanaNOAParser.With_viol_below, 0)

        def Angstrome(self, i:int=None):
            if i is None:
                return self.getTokens(CyanaNOAParser.Angstrome)
            else:
                return self.getToken(CyanaNOAParser.Angstrome, i)

        def With_viol_between(self):
            return self.getToken(CyanaNOAParser.With_viol_between, 0)

        def Float(self):
            return self.getToken(CyanaNOAParser.Float, 0)

        def And(self):
            return self.getToken(CyanaNOAParser.And, 0)

        def With_viol_above(self):
            return self.getToken(CyanaNOAParser.With_viol_above, 0)

        def With_diagonal(self):
            return self.getToken(CyanaNOAParser.With_diagonal, 0)

        def Cross_peaks(self):
            return self.getToken(CyanaNOAParser.Cross_peaks, 0)

        def With_off_diagonal(self):
            return self.getToken(CyanaNOAParser.With_off_diagonal, 0)

        def With_unique(self):
            return self.getToken(CyanaNOAParser.With_unique, 0)

        def With_short_range(self):
            return self.getToken(CyanaNOAParser.With_short_range, 0)

        def Short_range_ex(self):
            return self.getToken(CyanaNOAParser.Short_range_ex, 0)

        def With_medium_range(self):
            return self.getToken(CyanaNOAParser.With_medium_range, 0)

        def Medium_range_ex(self):
            return self.getToken(CyanaNOAParser.Medium_range_ex, 0)

        def With_long_range(self):
            return self.getToken(CyanaNOAParser.With_long_range, 0)

        def Long_range_ex(self):
            return self.getToken(CyanaNOAParser.Long_range_ex, 0)

        def getRuleIndex(self):
            return CyanaNOAParser.RULE_peak_stat

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPeak_stat" ):
                listener.enterPeak_stat(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPeak_stat" ):
                listener.exitPeak_stat(self)




    def peak_stat(self):

        localctx = CyanaNOAParser.Peak_statContext(self, self._ctx, self.state)
        self.enterRule(localctx, 22, self.RULE_peak_stat)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 151
            self.match(CyanaNOAParser.Peaks)
            self.state = 152
            self.match(CyanaNOAParser.Colon)
            self.state = 153
            self.match(CyanaNOAParser.Selected)
            self.state = 154
            self.match(CyanaNOAParser.Colon)
            self.state = 155
            self.match(CyanaNOAParser.Integer)
            self.state = 156
            self.match(CyanaNOAParser.Assigned)
            self.state = 157
            self.match(CyanaNOAParser.Colon)
            self.state = 158
            self.match(CyanaNOAParser.Integer)
            self.state = 159
            self.match(CyanaNOAParser.Unassigned)
            self.state = 160
            self.match(CyanaNOAParser.Colon)
            self.state = 161
            self.match(CyanaNOAParser.Integer)
            self.state = 162
            self.match(CyanaNOAParser.Without_possibility)
            self.state = 163
            self.match(CyanaNOAParser.Colon)
            self.state = 164
            self.match(CyanaNOAParser.Integer)
            self.state = 165
            self.match(CyanaNOAParser.With_viol_below)
            self.state = 166
            self.match(CyanaNOAParser.Angstrome)
            self.state = 167
            self.match(CyanaNOAParser.Colon)
            self.state = 168
            self.match(CyanaNOAParser.Integer)
            self.state = 169
            self.match(CyanaNOAParser.With_viol_between)
            self.state = 170
            self.match(CyanaNOAParser.Float)
            self.state = 171
            self.match(CyanaNOAParser.And)
            self.state = 172
            self.match(CyanaNOAParser.Angstrome)
            self.state = 173
            self.match(CyanaNOAParser.Colon)
            self.state = 174
            self.match(CyanaNOAParser.Integer)
            self.state = 175
            self.match(CyanaNOAParser.With_viol_above)
            self.state = 176
            self.match(CyanaNOAParser.Angstrome)
            self.state = 177
            self.match(CyanaNOAParser.Colon)
            self.state = 178
            self.match(CyanaNOAParser.Integer)
            self.state = 179
            self.match(CyanaNOAParser.With_diagonal)
            self.state = 180
            self.match(CyanaNOAParser.Colon)
            self.state = 181
            self.match(CyanaNOAParser.Integer)
            self.state = 182
            self.match(CyanaNOAParser.Cross_peaks)
            self.state = 183
            self.match(CyanaNOAParser.Colon)
            self.state = 184
            self.match(CyanaNOAParser.With_off_diagonal)
            self.state = 185
            self.match(CyanaNOAParser.Colon)
            self.state = 186
            self.match(CyanaNOAParser.Integer)
            self.state = 187
            self.match(CyanaNOAParser.With_unique)
            self.state = 188
            self.match(CyanaNOAParser.Colon)
            self.state = 189
            self.match(CyanaNOAParser.Integer)
            self.state = 190
            self.match(CyanaNOAParser.With_short_range)
            self.state = 191
            self.match(CyanaNOAParser.Short_range_ex)
            self.state = 192
            self.match(CyanaNOAParser.Colon)
            self.state = 193
            self.match(CyanaNOAParser.Integer)
            self.state = 194
            self.match(CyanaNOAParser.With_medium_range)
            self.state = 195
            self.match(CyanaNOAParser.Medium_range_ex)
            self.state = 196
            self.match(CyanaNOAParser.Colon)
            self.state = 197
            self.match(CyanaNOAParser.Integer)
            self.state = 198
            self.match(CyanaNOAParser.With_long_range)
            self.state = 199
            self.match(CyanaNOAParser.Long_range_ex)
            self.state = 200
            self.match(CyanaNOAParser.Colon)
            self.state = 201
            self.match(CyanaNOAParser.Integer)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx








# Generated from SparkyPKParser.g4 by ANTLR 4.13.0
# encoding: utf-8
from antlr4 import *
from io import StringIO
import sys
if sys.version_info[1] > 5:
	from typing import TextIO
else:
	from typing.io import TextIO

def serializedATN():
    return [
        4,1,25,125,2,0,7,0,2,1,7,1,2,2,7,2,2,3,7,3,2,4,7,4,2,5,7,5,2,6,7,
        6,2,7,7,7,1,0,3,0,18,8,0,1,0,1,0,1,0,4,0,23,8,0,11,0,12,0,24,1,0,
        4,0,28,8,0,11,0,12,0,29,1,0,4,0,33,8,0,11,0,12,0,34,5,0,37,8,0,10,
        0,12,0,40,9,0,1,0,1,0,1,1,1,1,1,1,1,1,3,1,48,8,1,1,1,3,1,51,8,1,
        1,1,1,1,3,1,55,8,1,1,1,3,1,58,8,1,1,1,1,1,1,2,1,2,1,2,3,2,65,8,2,
        1,2,3,2,68,8,2,1,2,1,2,3,2,72,8,2,1,2,3,2,75,8,2,1,2,1,2,4,2,79,
        8,2,11,2,12,2,80,1,3,1,3,1,3,1,3,4,3,87,8,3,11,3,12,3,88,1,3,1,3,
        1,4,1,4,1,4,1,4,1,4,4,4,98,8,4,11,4,12,4,99,1,4,1,4,1,5,1,5,1,5,
        1,5,1,5,1,5,4,5,110,8,5,11,5,12,5,111,1,5,1,5,1,6,4,6,117,8,6,11,
        6,12,6,118,1,6,1,6,1,7,1,7,1,7,0,0,8,0,2,4,6,8,10,12,14,0,1,1,0,
        3,5,138,0,17,1,0,0,0,2,43,1,0,0,0,4,61,1,0,0,0,6,82,1,0,0,0,8,92,
        1,0,0,0,10,103,1,0,0,0,12,116,1,0,0,0,14,122,1,0,0,0,16,18,5,13,
        0,0,17,16,1,0,0,0,17,18,1,0,0,0,18,38,1,0,0,0,19,37,3,2,1,0,20,37,
        3,4,2,0,21,23,3,6,3,0,22,21,1,0,0,0,23,24,1,0,0,0,24,22,1,0,0,0,
        24,25,1,0,0,0,25,37,1,0,0,0,26,28,3,8,4,0,27,26,1,0,0,0,28,29,1,
        0,0,0,29,27,1,0,0,0,29,30,1,0,0,0,30,37,1,0,0,0,31,33,3,10,5,0,32,
        31,1,0,0,0,33,34,1,0,0,0,34,32,1,0,0,0,34,35,1,0,0,0,35,37,1,0,0,
        0,36,19,1,0,0,0,36,20,1,0,0,0,36,22,1,0,0,0,36,27,1,0,0,0,36,32,
        1,0,0,0,37,40,1,0,0,0,38,36,1,0,0,0,38,39,1,0,0,0,39,41,1,0,0,0,
        40,38,1,0,0,0,41,42,5,0,0,1,42,1,1,0,0,0,43,44,5,1,0,0,44,45,5,17,
        0,0,45,47,5,18,0,0,46,48,5,19,0,0,47,46,1,0,0,0,47,48,1,0,0,0,48,
        50,1,0,0,0,49,51,5,20,0,0,50,49,1,0,0,0,50,51,1,0,0,0,51,52,1,0,
        0,0,52,54,5,21,0,0,53,55,5,22,0,0,54,53,1,0,0,0,54,55,1,0,0,0,55,
        57,1,0,0,0,56,58,5,23,0,0,57,56,1,0,0,0,57,58,1,0,0,0,58,59,1,0,
        0,0,59,60,5,25,0,0,60,3,1,0,0,0,61,62,5,2,0,0,62,64,5,18,0,0,63,
        65,5,19,0,0,64,63,1,0,0,0,64,65,1,0,0,0,65,67,1,0,0,0,66,68,5,20,
        0,0,67,66,1,0,0,0,67,68,1,0,0,0,68,69,1,0,0,0,69,71,5,21,0,0,70,
        72,5,22,0,0,71,70,1,0,0,0,71,72,1,0,0,0,72,74,1,0,0,0,73,75,5,23,
        0,0,74,73,1,0,0,0,74,75,1,0,0,0,75,76,1,0,0,0,76,78,5,25,0,0,77,
        79,3,12,6,0,78,77,1,0,0,0,79,80,1,0,0,0,80,78,1,0,0,0,80,81,1,0,
        0,0,81,5,1,0,0,0,82,83,5,9,0,0,83,84,5,4,0,0,84,86,5,4,0,0,85,87,
        3,14,7,0,86,85,1,0,0,0,87,88,1,0,0,0,88,86,1,0,0,0,88,89,1,0,0,0,
        89,90,1,0,0,0,90,91,5,13,0,0,91,7,1,0,0,0,92,93,5,10,0,0,93,94,5,
        4,0,0,94,95,5,4,0,0,95,97,5,4,0,0,96,98,3,14,7,0,97,96,1,0,0,0,98,
        99,1,0,0,0,99,97,1,0,0,0,99,100,1,0,0,0,100,101,1,0,0,0,101,102,
        5,13,0,0,102,9,1,0,0,0,103,104,5,11,0,0,104,105,5,4,0,0,105,106,
        5,4,0,0,106,107,5,4,0,0,107,109,5,4,0,0,108,110,3,14,7,0,109,108,
        1,0,0,0,110,111,1,0,0,0,111,109,1,0,0,0,111,112,1,0,0,0,112,113,
        1,0,0,0,113,114,5,13,0,0,114,11,1,0,0,0,115,117,3,14,7,0,116,115,
        1,0,0,0,117,118,1,0,0,0,118,116,1,0,0,0,118,119,1,0,0,0,119,120,
        1,0,0,0,120,121,5,13,0,0,121,13,1,0,0,0,122,123,7,0,0,0,123,15,1,
        0,0,0,19,17,24,29,34,36,38,47,50,54,57,64,67,71,74,80,88,99,111,
        118
    ]

class SparkyPKParser ( Parser ):

    grammarFileName = "SparkyPKParser.g4"

    atn = ATNDeserializer().deserialize(serializedATN())

    decisionsToDFA = [ DFA(ds, i) for i, ds in enumerate(atn.decisionToState) ]

    sharedContextCache = PredictionContextCache()

    literalNames = [ "<INVALID>", "'Assignment'", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "'w2'", "'w3'", "'w4'", "<INVALID>", 
                     "'Volume'", "'S/N'" ]

    symbolicNames = [ "<INVALID>", "Assignment", "W1", "Integer", "Float", 
                      "Real", "SHARP_COMMENT", "EXCLM_COMMENT", "SMCLN_COMMENT", 
                      "Assignment_2d_ex", "Assignment_3d_ex", "Assignment_4d_ex", 
                      "SPACE", "RETURN", "ENCLOSE_COMMENT", "SECTION_COMMENT", 
                      "LINE_COMMENT", "W1_LA", "W2_LA", "W3_LA", "W4_LA", 
                      "Height_LA", "Volume_LA", "S_N_LA", "SPACE_LA", "RETURN_LA" ]

    RULE_sparky_pk = 0
    RULE_data_label = 1
    RULE_data_label_wo_assign = 2
    RULE_peak_2d = 3
    RULE_peak_3d = 4
    RULE_peak_4d = 5
    RULE_peak_wo_assign = 6
    RULE_number = 7

    ruleNames =  [ "sparky_pk", "data_label", "data_label_wo_assign", "peak_2d", 
                   "peak_3d", "peak_4d", "peak_wo_assign", "number" ]

    EOF = Token.EOF
    Assignment=1
    W1=2
    Integer=3
    Float=4
    Real=5
    SHARP_COMMENT=6
    EXCLM_COMMENT=7
    SMCLN_COMMENT=8
    Assignment_2d_ex=9
    Assignment_3d_ex=10
    Assignment_4d_ex=11
    SPACE=12
    RETURN=13
    ENCLOSE_COMMENT=14
    SECTION_COMMENT=15
    LINE_COMMENT=16
    W1_LA=17
    W2_LA=18
    W3_LA=19
    W4_LA=20
    Height_LA=21
    Volume_LA=22
    S_N_LA=23
    SPACE_LA=24
    RETURN_LA=25

    def __init__(self, input:TokenStream, output:TextIO = sys.stdout):
        super().__init__(input, output)
        self.checkVersion("4.13.0")
        self._interp = ParserATNSimulator(self, self.atn, self.decisionsToDFA, self.sharedContextCache)
        self._predicates = None




    class Sparky_pkContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def EOF(self):
            return self.getToken(SparkyPKParser.EOF, 0)

        def RETURN(self):
            return self.getToken(SparkyPKParser.RETURN, 0)

        def data_label(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(SparkyPKParser.Data_labelContext)
            else:
                return self.getTypedRuleContext(SparkyPKParser.Data_labelContext,i)


        def data_label_wo_assign(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(SparkyPKParser.Data_label_wo_assignContext)
            else:
                return self.getTypedRuleContext(SparkyPKParser.Data_label_wo_assignContext,i)


        def peak_2d(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(SparkyPKParser.Peak_2dContext)
            else:
                return self.getTypedRuleContext(SparkyPKParser.Peak_2dContext,i)


        def peak_3d(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(SparkyPKParser.Peak_3dContext)
            else:
                return self.getTypedRuleContext(SparkyPKParser.Peak_3dContext,i)


        def peak_4d(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(SparkyPKParser.Peak_4dContext)
            else:
                return self.getTypedRuleContext(SparkyPKParser.Peak_4dContext,i)


        def getRuleIndex(self):
            return SparkyPKParser.RULE_sparky_pk

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSparky_pk" ):
                listener.enterSparky_pk(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSparky_pk" ):
                listener.exitSparky_pk(self)




    def sparky_pk(self):

        localctx = SparkyPKParser.Sparky_pkContext(self, self._ctx, self.state)
        self.enterRule(localctx, 0, self.RULE_sparky_pk)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 17
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==13:
                self.state = 16
                self.match(SparkyPKParser.RETURN)


            self.state = 38
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while (((_la) & ~0x3f) == 0 and ((1 << _la) & 3590) != 0):
                self.state = 36
                self._errHandler.sync(self)
                token = self._input.LA(1)
                if token in [1]:
                    self.state = 19
                    self.data_label()
                    pass
                elif token in [2]:
                    self.state = 20
                    self.data_label_wo_assign()
                    pass
                elif token in [9]:
                    self.state = 22 
                    self._errHandler.sync(self)
                    _alt = 1
                    while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                        if _alt == 1:
                            self.state = 21
                            self.peak_2d()

                        else:
                            raise NoViableAltException(self)
                        self.state = 24 
                        self._errHandler.sync(self)
                        _alt = self._interp.adaptivePredict(self._input,1,self._ctx)

                    pass
                elif token in [10]:
                    self.state = 27 
                    self._errHandler.sync(self)
                    _alt = 1
                    while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                        if _alt == 1:
                            self.state = 26
                            self.peak_3d()

                        else:
                            raise NoViableAltException(self)
                        self.state = 29 
                        self._errHandler.sync(self)
                        _alt = self._interp.adaptivePredict(self._input,2,self._ctx)

                    pass
                elif token in [11]:
                    self.state = 32 
                    self._errHandler.sync(self)
                    _alt = 1
                    while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                        if _alt == 1:
                            self.state = 31
                            self.peak_4d()

                        else:
                            raise NoViableAltException(self)
                        self.state = 34 
                        self._errHandler.sync(self)
                        _alt = self._interp.adaptivePredict(self._input,3,self._ctx)

                    pass
                else:
                    raise NoViableAltException(self)

                self.state = 40
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 41
            self.match(SparkyPKParser.EOF)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Data_labelContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Assignment(self):
            return self.getToken(SparkyPKParser.Assignment, 0)

        def W1_LA(self):
            return self.getToken(SparkyPKParser.W1_LA, 0)

        def W2_LA(self):
            return self.getToken(SparkyPKParser.W2_LA, 0)

        def Height_LA(self):
            return self.getToken(SparkyPKParser.Height_LA, 0)

        def RETURN_LA(self):
            return self.getToken(SparkyPKParser.RETURN_LA, 0)

        def W3_LA(self):
            return self.getToken(SparkyPKParser.W3_LA, 0)

        def W4_LA(self):
            return self.getToken(SparkyPKParser.W4_LA, 0)

        def Volume_LA(self):
            return self.getToken(SparkyPKParser.Volume_LA, 0)

        def S_N_LA(self):
            return self.getToken(SparkyPKParser.S_N_LA, 0)

        def getRuleIndex(self):
            return SparkyPKParser.RULE_data_label

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterData_label" ):
                listener.enterData_label(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitData_label" ):
                listener.exitData_label(self)




    def data_label(self):

        localctx = SparkyPKParser.Data_labelContext(self, self._ctx, self.state)
        self.enterRule(localctx, 2, self.RULE_data_label)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 43
            self.match(SparkyPKParser.Assignment)
            self.state = 44
            self.match(SparkyPKParser.W1_LA)
            self.state = 45
            self.match(SparkyPKParser.W2_LA)
            self.state = 47
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==19:
                self.state = 46
                self.match(SparkyPKParser.W3_LA)


            self.state = 50
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==20:
                self.state = 49
                self.match(SparkyPKParser.W4_LA)


            self.state = 52
            self.match(SparkyPKParser.Height_LA)
            self.state = 54
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==22:
                self.state = 53
                self.match(SparkyPKParser.Volume_LA)


            self.state = 57
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==23:
                self.state = 56
                self.match(SparkyPKParser.S_N_LA)


            self.state = 59
            self.match(SparkyPKParser.RETURN_LA)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Data_label_wo_assignContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def W1(self):
            return self.getToken(SparkyPKParser.W1, 0)

        def W2_LA(self):
            return self.getToken(SparkyPKParser.W2_LA, 0)

        def Height_LA(self):
            return self.getToken(SparkyPKParser.Height_LA, 0)

        def RETURN_LA(self):
            return self.getToken(SparkyPKParser.RETURN_LA, 0)

        def W3_LA(self):
            return self.getToken(SparkyPKParser.W3_LA, 0)

        def W4_LA(self):
            return self.getToken(SparkyPKParser.W4_LA, 0)

        def Volume_LA(self):
            return self.getToken(SparkyPKParser.Volume_LA, 0)

        def S_N_LA(self):
            return self.getToken(SparkyPKParser.S_N_LA, 0)

        def peak_wo_assign(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(SparkyPKParser.Peak_wo_assignContext)
            else:
                return self.getTypedRuleContext(SparkyPKParser.Peak_wo_assignContext,i)


        def getRuleIndex(self):
            return SparkyPKParser.RULE_data_label_wo_assign

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterData_label_wo_assign" ):
                listener.enterData_label_wo_assign(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitData_label_wo_assign" ):
                listener.exitData_label_wo_assign(self)




    def data_label_wo_assign(self):

        localctx = SparkyPKParser.Data_label_wo_assignContext(self, self._ctx, self.state)
        self.enterRule(localctx, 4, self.RULE_data_label_wo_assign)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 61
            self.match(SparkyPKParser.W1)
            self.state = 62
            self.match(SparkyPKParser.W2_LA)
            self.state = 64
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==19:
                self.state = 63
                self.match(SparkyPKParser.W3_LA)


            self.state = 67
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==20:
                self.state = 66
                self.match(SparkyPKParser.W4_LA)


            self.state = 69
            self.match(SparkyPKParser.Height_LA)
            self.state = 71
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==22:
                self.state = 70
                self.match(SparkyPKParser.Volume_LA)


            self.state = 74
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==23:
                self.state = 73
                self.match(SparkyPKParser.S_N_LA)


            self.state = 76
            self.match(SparkyPKParser.RETURN_LA)
            self.state = 78 
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while True:
                self.state = 77
                self.peak_wo_assign()
                self.state = 80 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if not ((((_la) & ~0x3f) == 0 and ((1 << _la) & 56) != 0)):
                    break

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Peak_2dContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Assignment_2d_ex(self):
            return self.getToken(SparkyPKParser.Assignment_2d_ex, 0)

        def Float(self, i:int=None):
            if i is None:
                return self.getTokens(SparkyPKParser.Float)
            else:
                return self.getToken(SparkyPKParser.Float, i)

        def RETURN(self):
            return self.getToken(SparkyPKParser.RETURN, 0)

        def number(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(SparkyPKParser.NumberContext)
            else:
                return self.getTypedRuleContext(SparkyPKParser.NumberContext,i)


        def getRuleIndex(self):
            return SparkyPKParser.RULE_peak_2d

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPeak_2d" ):
                listener.enterPeak_2d(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPeak_2d" ):
                listener.exitPeak_2d(self)




    def peak_2d(self):

        localctx = SparkyPKParser.Peak_2dContext(self, self._ctx, self.state)
        self.enterRule(localctx, 6, self.RULE_peak_2d)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 82
            self.match(SparkyPKParser.Assignment_2d_ex)
            self.state = 83
            self.match(SparkyPKParser.Float)
            self.state = 84
            self.match(SparkyPKParser.Float)
            self.state = 86 
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while True:
                self.state = 85
                self.number()
                self.state = 88 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if not ((((_la) & ~0x3f) == 0 and ((1 << _la) & 56) != 0)):
                    break

            self.state = 90
            self.match(SparkyPKParser.RETURN)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Peak_3dContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Assignment_3d_ex(self):
            return self.getToken(SparkyPKParser.Assignment_3d_ex, 0)

        def Float(self, i:int=None):
            if i is None:
                return self.getTokens(SparkyPKParser.Float)
            else:
                return self.getToken(SparkyPKParser.Float, i)

        def RETURN(self):
            return self.getToken(SparkyPKParser.RETURN, 0)

        def number(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(SparkyPKParser.NumberContext)
            else:
                return self.getTypedRuleContext(SparkyPKParser.NumberContext,i)


        def getRuleIndex(self):
            return SparkyPKParser.RULE_peak_3d

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPeak_3d" ):
                listener.enterPeak_3d(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPeak_3d" ):
                listener.exitPeak_3d(self)




    def peak_3d(self):

        localctx = SparkyPKParser.Peak_3dContext(self, self._ctx, self.state)
        self.enterRule(localctx, 8, self.RULE_peak_3d)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 92
            self.match(SparkyPKParser.Assignment_3d_ex)
            self.state = 93
            self.match(SparkyPKParser.Float)
            self.state = 94
            self.match(SparkyPKParser.Float)
            self.state = 95
            self.match(SparkyPKParser.Float)
            self.state = 97 
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while True:
                self.state = 96
                self.number()
                self.state = 99 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if not ((((_la) & ~0x3f) == 0 and ((1 << _la) & 56) != 0)):
                    break

            self.state = 101
            self.match(SparkyPKParser.RETURN)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Peak_4dContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Assignment_4d_ex(self):
            return self.getToken(SparkyPKParser.Assignment_4d_ex, 0)

        def Float(self, i:int=None):
            if i is None:
                return self.getTokens(SparkyPKParser.Float)
            else:
                return self.getToken(SparkyPKParser.Float, i)

        def RETURN(self):
            return self.getToken(SparkyPKParser.RETURN, 0)

        def number(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(SparkyPKParser.NumberContext)
            else:
                return self.getTypedRuleContext(SparkyPKParser.NumberContext,i)


        def getRuleIndex(self):
            return SparkyPKParser.RULE_peak_4d

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPeak_4d" ):
                listener.enterPeak_4d(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPeak_4d" ):
                listener.exitPeak_4d(self)




    def peak_4d(self):

        localctx = SparkyPKParser.Peak_4dContext(self, self._ctx, self.state)
        self.enterRule(localctx, 10, self.RULE_peak_4d)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 103
            self.match(SparkyPKParser.Assignment_4d_ex)
            self.state = 104
            self.match(SparkyPKParser.Float)
            self.state = 105
            self.match(SparkyPKParser.Float)
            self.state = 106
            self.match(SparkyPKParser.Float)
            self.state = 107
            self.match(SparkyPKParser.Float)
            self.state = 109 
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while True:
                self.state = 108
                self.number()
                self.state = 111 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if not ((((_la) & ~0x3f) == 0 and ((1 << _la) & 56) != 0)):
                    break

            self.state = 113
            self.match(SparkyPKParser.RETURN)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Peak_wo_assignContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def RETURN(self):
            return self.getToken(SparkyPKParser.RETURN, 0)

        def number(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(SparkyPKParser.NumberContext)
            else:
                return self.getTypedRuleContext(SparkyPKParser.NumberContext,i)


        def getRuleIndex(self):
            return SparkyPKParser.RULE_peak_wo_assign

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPeak_wo_assign" ):
                listener.enterPeak_wo_assign(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPeak_wo_assign" ):
                listener.exitPeak_wo_assign(self)




    def peak_wo_assign(self):

        localctx = SparkyPKParser.Peak_wo_assignContext(self, self._ctx, self.state)
        self.enterRule(localctx, 12, self.RULE_peak_wo_assign)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 116 
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while True:
                self.state = 115
                self.number()
                self.state = 118 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if not ((((_la) & ~0x3f) == 0 and ((1 << _la) & 56) != 0)):
                    break

            self.state = 120
            self.match(SparkyPKParser.RETURN)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class NumberContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Real(self):
            return self.getToken(SparkyPKParser.Real, 0)

        def Float(self):
            return self.getToken(SparkyPKParser.Float, 0)

        def Integer(self):
            return self.getToken(SparkyPKParser.Integer, 0)

        def getRuleIndex(self):
            return SparkyPKParser.RULE_number

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterNumber" ):
                listener.enterNumber(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitNumber" ):
                listener.exitNumber(self)




    def number(self):

        localctx = SparkyPKParser.NumberContext(self, self._ctx, self.state)
        self.enterRule(localctx, 14, self.RULE_number)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 122
            _la = self._input.LA(1)
            if not((((_la) & ~0x3f) == 0 and ((1 << _la) & 56) != 0)):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx





